##Bias correction for multi-model composite
##Note: this script is computationally demanding
##It is likely taking over 7 hours in computation
##Citation: Chang, K.-L., Cooper, O.R., West, J.J., Serre, M.L., Schultz, M.G.,
##Lin, M., Marécal, V., Josse, B., Deushi, M., Sudo, K., Liu, J., and Keller, C.A.:
##A new method (M3Fusion v1) for combining observations and multiple model output
##for an improved estimate of the global surface ozone distribution,
##Geosci. Model Dev., 2019.

col=c(rgb(0.2081, 0.1663, 0.5292),
rgb(0.3961, 0.3176, 0.8000),
rgb(0.0123, 0.4213, 0.8802),
rgb(0.4941, 0.7647, 0.8980),
rgb(0.1157, 0.7022, 0.6843),
rgb(0.5216, 0.6980, 0.1725),
rgb(0.9968, 0.7513, 0.2325),
rgb(1.0000, 0.4863, 0.0000),
rgb(0.8000, 0.3176, 0.3176),
rgb(0.6980, 0.1725, 0.1725))
N=10

library(dichromat)
col2=colorschemes$DarkRedtoBlue.18
col2=c('blue 4',col2,'red 4')
 
library(fields)
library(mgcv)
library(limSolve)
library(rworldmap)
newworld =getMap(resolution = "li")

z = read.table("dma8_output.txt",header=F,sep=" ")
names(z)=c('lon','lat','chaser','geosccm','gfdlam3','gmaonr','mocage','mriesm','spde')
z=cbind(1, z)


##############################################################################################

#####AFRICA
#cy1=z[z$lon>=-20,]
#cy2=cy1[cy1$lon<=30,]
#cy3=cy2[cy2$lat>=10,]
#cy41=cy3[cy3$lat<=37,]
#cy1=z[z$lon>=-20,]
#cy2=cy1[cy1$lon<=60,]
#cy3=cy2[cy2$lat>=-40,]
#cy42=cy3[cy3$lat<=10,]
cy1=z[z$lon>=-20,]
cy2=cy1[cy1$lon<=35,]
cy3=cy2[cy2$lat>=10,]
cy41=cy3[cy3$lat<=37,]
cy1=z[z$lon>=-20,]
cy2=cy1[cy1$lon<=60,]
cy3=cy2[cy2$lat>=-40,]
cy42=cy3[cy3$lat<=15,]
afr=rbind(cy41, cy42)
afr=na.omit(afr)

X=as.matrix(afr[,4:9])
Y=afr[,10]
#qp=lsei(A=cbind(X, 1), B=Y, E=c(rep(1,6),0), F=1, G=diag(7), H=rep(0,7))
qp=lsei(A=cbind(X, 1), B=Y, E=c(rep(1,6),0), F=1, G=rbind(cbind(diag(6),0),0), H=rep(0,7))
res=cbind(X,1)%*%qp$X-Y
afr$res=res


#####E ASIA
#cy1=z[z$lon>=90,]
#cy2=cy1[cy1$lon<=165,]
#cy3=cy2[cy2$lat>=10,]
#cy4=cy3[cy3$lat<=50,]
cy1=z[z$lon>=85,]
cy2=cy1[cy1$lon<=165,]
cy3=cy2[cy2$lat>=5,]
cy4=cy3[cy3$lat<=55,]

ea=na.omit(cy4)
X=as.matrix(ea[,4:9])
Y=ea[,10]
qp=lsei(A=cbind(X, 1), B=Y, E=c(rep(1,6),0), F=1, G=rbind(cbind(diag(6),0),0), H=rep(0,7))
res=cbind(X,1)%*%qp$X-Y
ea$res=res


#####S C ASIA
#cy1=z[z$lon>=30,]
#cy2=cy1[cy1$lon<=90,]
#cy3=cy2[cy2$lat>=10,]
#cy4=cy3[cy3$lat<=50,]
cy1=z[z$lon>=25,]
cy2=cy1[cy1$lon<=95,]
cy3=cy2[cy2$lat>=5,]
cy4=cy3[cy3$lat<=55,]

sca=na.omit(cy4)
X=as.matrix(sca[,4:9])
Y=sca[,10]
qp=lsei(A=cbind(X, 1), B=Y, E=c(rep(1,6),0), F=1, G=rbind(cbind(diag(6),0),0), H=rep(0,7))
res=cbind(X,1)%*%qp$X-Y
sca$res=res


#####RUSSIA
#cy1=z[z$lon>=30,]
#cy2=cy1[cy1$lon<=180,]
#cy3=cy2[cy2$lat>=50,]
#cy4=cy3[cy3$lat<=75,]
cy1=z[z$lon>=25,]
cy2=cy1[cy1$lon<=180,]
cy3=cy2[cy2$lat>=45,]
cy41=cy3[cy3$lat<=75,]
cy1=z[z$lon>=-180,]
cy2=cy1[cy1$lon<=-170,]
cy3=cy2[cy2$lat>=50,]
cy42=cy3[cy3$lat<=75,]
rus=rbind(cy41, cy42)
rus=na.omit(rus)

X=as.matrix(rus[,4:9])
Y=rus[,10]
qp=lsei(A=cbind(X, 1), B=Y, E=c(rep(1,6),0), F=1, G=rbind(cbind(diag(6),0),0), H=rep(0,7))
res=cbind(X,1)%*%qp$X-Y
rus$res=res


#####EUROPE
#cy1=z[z$lon>=-23,]
#cy2=cy1[cy1$lon<=30,]
#cy3=cy2[cy2$lat>=37,]
#cy4=cy3[cy3$lat<=75,]
cy1=z[z$lon>=-23,]
cy2=cy1[cy1$lon<=35,]
cy3=cy2[cy2$lat>=37,]
cy4=cy3[cy3$lat<=75,]

eur=na.omit(cy4)
X=as.matrix(eur[,4:9])
Y=eur[,10]
qp=lsei(A=cbind(X, 1), B=Y, E=c(rep(1,6),0), F=1, G=rbind(cbind(diag(6),0),0), H=rep(0,7))
res=cbind(X,1)%*%qp$X-Y
eur$res=res


##### OCEANIA 
cy1=z[z$lon>=90,]
cy2=cy1[cy1$lon<=180,]
cy3=cy2[cy2$lat>=-50,]
cy41=cy3[cy3$lat<=10,]
cy1=z[z$lon>=-180,]
cy2=cy1[cy1$lon<=-170,]
cy3=cy2[cy2$lat>=-50,]
cy42=cy3[cy3$lat<=10,]

oce=rbind(cy41, cy42)
oce=na.omit(oce)
X=as.matrix(oce[,4:9])
Y=oce[,10] 
qp=lsei(A=cbind(X, 1), B=Y, E=c(rep(1,6),0), F=1, G=rbind(cbind(diag(6),0),0), H=rep(0,7))
res=cbind(X,1)%*%qp$X-Y
oce$res=res


##### N AMERICA
cy1=z[z$lon>=-168,]
cy2=cy1[cy1$lon<=-55,]
cy3=cy2[cy2$lat>=20,]
cy41=cy3[cy3$lat<=75,]
cy1=z[z$lon>=-168,]
cy2=cy1[cy1$lon<=-90,]
cy3=cy2[cy2$lat>=15,]
cy42=cy3[cy3$lat<=20,]

nam=rbind(cy41, cy42)
nam=na.omit(nam)
X=as.matrix(nam[,4:9])
Y=nam[,10] 
qp=lsei(A=cbind(X, 1), B=Y, E=c(rep(1,6),0), F=1, G=rbind(cbind(diag(6),0),0), H=rep(0,7))
res=cbind(X,1)%*%qp$X-Y
nam$res=res


##### S AMERICA
cy1=z[z$lon>=-90,]
cy2=cy1[cy1$lon<=-30,]
cy3=cy2[cy2$lat>=-60,]
cy4=cy3[cy3$lat<=20,]

sam=na.omit(cy4)
X=as.matrix(sam[,4:9])
Y=sam[,10]
qp=lsei(A=cbind(X, 1), B=Y, E=c(rep(1,6),0), F=1, G=rbind(cbind(diag(6),0),0), H=rep(0,7))
res=cbind(X,1)%*%qp$X-Y
sam$res=res

comb=rbind(afr,ea,sca,rus,eur,oce,nam,sam)
comb$composite=unlist(comb$res + comb$spde)
comb=aggregate(cbind(chaser,geosccm,gfdlam3,gmaonr,mocage,mriesm,spde,res, composite)~lon+lat,data=comb,FUN=mean)
names(comb)=c('lon','lat','chaser','geosccm','gfdlam3','gmaonr','mocage','mriesm','spde','res','composite')


####################################################################################

##smooth out disjoint region 
####################################################################################

sm=comb[comb$lon<=180 & comb$lon>=30 & comb$lat<=70 & comb$lat>=50,]

mod = gam(composite ~ s(lon,lat, bs="gp", k=180), data=sm, method="REML", na.action='na.omit')

#quilt.plot(sm$lon, sm$lat, mod$fitted.values, xlim=c(30,180),ylim=c(50,70), nx=300, ny=40)

comb[comb$lon<=180 & comb$lon>=30 & comb$lat<=70 & comb$lat>=50,]$composite= mod$fitted.values


sm=comb[comb$lon<=105 & comb$lon>=75 & comb$lat<=60 & comb$lat>=0,]

mod = gam(composite ~ s(lon,lat, bs="gp", k=180), data=sm, method="REML", na.action='na.omit')

comb[comb$lon<=105 & comb$lon>=75 & comb$lat<=60 & comb$lat>=0,]$composite= mod$fitted.values


sm=comb[comb$lon<=40 & comb$lon>=30 & comb$lat<=20 & comb$lat>=0,]

mod = gam(composite ~ s(lon,lat, bs="gp", k=90), data=sm, method="REML", na.action='na.omit')

comb[comb$lon<=40 & comb$lon>=30 & comb$lat<=20 & comb$lat>=0,]$composite= mod$fitted.values

####################################################################################

comb$res=comb$composite-comb$spde
#quilt.plot(comb$lon, comb$lat, comb$composite, xlim=c(-180,180),ylim=c(-75,75), nx=180, ny=75)
#world(add=T)

mod2 = gam(res ~ s(lon,lat, bs="gp",k=270), data=comb, method="REML", na.action='na.omit')
summary(mod2)

comb$fbias=mod2$fitted.values
comb$fcorr= unlist(comb$res + comb$spde - mod2$fitted.values)
write.table(comb, "dma8_combined.txt",sep = " ",row.names = F, col.names = F)


################################################################
####the following script can be ran separately for saving memory
################################################################

mdt = read.table("dma8_combined.txt",header=F,sep=" ")
names(mdt)=c('lon','lat','chaser','geosccm','gfdlam3','gmaonr','mocage','mriesm','spde', 'res', 'composite', 'fbias','fcorr')
comb=mdt

cx=seq(-180,180,by=0.125)
cy=seq(-60,75,by=0.125)
sg=expand.grid(lon=cx, lat=cy)
mdt = merge(comb, sg,by=c("lat","lon"), all=T, sort=T)


dt1 = read.table("toar-6mo8hr-dma8-2008.txt",header=F,sep=" ")
dt2 = read.table("toar-6mo8hr-dma8-2009.txt",header=F,sep=" ")
dt3 = read.table("toar-6mo8hr-dma8-2010.txt",header=F,sep=" ")
dt4 = read.table("toar-6mo8hr-dma8-2011.txt",header=F,sep=" ")
dt5 = read.table("toar-6mo8hr-dma8-2012.txt",header=F,sep=" ")
dt6 = read.table("toar-6mo8hr-dma8-2013.txt",header=F,sep=" ")
dt7 = read.table("toar-6mo8hr-dma8-2014.txt",header=F,sep=" ")
dt=rbind(dt1,dt2,dt3,dt4,dt5,dt6,dt7)
dt=na.omit(dt)
colnames(dt)=c('id', 'lon', 'lat', 'avgdma8epax')
dt=aggregate(avgdma8epax~lon+lat,data=dt,FUN=mean, na.rm=F)

obj=list(x=cx, y=cy, z=matrix(mdt$composite, nrow=length(cx)))
loc=cbind(x=dt$lon, y=dt$lat)
interp.surface(obj, loc)-> back1
sqrt(mean((back1-dt$avgdma8epax)^2, na.rm=T))

obj=list(x=cx, y=cy, z=matrix(mdt$spde, nrow=length(cx)))
loc=cbind(x=dt$lon, y=dt$lat)
interp.surface(obj, loc)-> back2
sqrt(mean((back2-dt$avgdma8epax)^2, na.rm=T))


obj=list(x=cx, y=cy, z=matrix(mdt$chaser, nrow=length(cx)))
ch=interp.surface(obj, loc)-dt$avgdma8epax
obj=list(x=cx, y=cy, z=matrix(mdt$geosccm, nrow=length(cx)))
go=interp.surface(obj, loc)-dt$avgdma8epax
obj=list(x=cx, y=cy, z=matrix(mdt$gfdlam3, nrow=length(cx)))
gf=interp.surface(obj, loc)-dt$avgdma8epax
obj=list(x=cx, y=cy, z=matrix(mdt$gmaonr, nrow=length(cx)))
gm=interp.surface(obj, loc)-dt$avgdma8epax
obj=list(x=cx, y=cy, z=matrix(mdt$mocage, nrow=length(cx)))
mo=interp.surface(obj, loc)-dt$avgdma8epax
obj=list(x=cx, y=cy, z=matrix(mdt$mriesm, nrow=length(cx)))
mr=interp.surface(obj, loc)-dt$avgdma8epax
obj=list(x=cx, y=cy, z=matrix(mdt$spde, nrow=length(cx)))
sp=interp.surface(obj, loc)-dt$avgdma8epax
obj=list(x=cx, y=cy, z=matrix(mdt$composite, nrow=length(cx)))
mu=interp.surface(obj, loc)-dt$avgdma8epax
obj=list(x=cx, y=cy, z=matrix(mdt$fcorr, nrow=length(cx)))
co=interp.surface(obj, loc)-dt$avgdma8epax

mean(ch,na.rm=T)
sd(ch,na.rm=T)
mean(go,na.rm=T)
sd(go,na.rm=T)
mean(gf,na.rm=T)
sd(gf,na.rm=T)
mean(gm,na.rm=T)
sd(gm,na.rm=T)
mean(mo,na.rm=T)
sd(mo,na.rm=T)
mean(mr,na.rm=T)
sd(mr,na.rm=T)
mean(sp,na.rm=T)
sd(sp,na.rm=T)
mean(mu,na.rm=T)
sd(mu,na.rm=T)
mean(co,na.rm=T)
sd(co,na.rm=T)
sqrt(mean(ch^2, na.rm=T))
sqrt(mean(go^2, na.rm=T))
sqrt(mean(gf^2, na.rm=T))
sqrt(mean(gm^2, na.rm=T))
sqrt(mean(mo^2, na.rm=T))
sqrt(mean(mr^2, na.rm=T))
sqrt(mean(sp^2, na.rm=T))
sqrt(mean(mu^2, na.rm=T))
sqrt(mean(co^2, na.rm=T))

obj=list(x=cx, y=cy, z=matrix((mdt$chaser+mdt$geosccm+mdt$gfdlam3+mdt$gmaonr+mdt$mriesm+mdt$mocage)/6, nrow=length(cx)))
mm1=interp.surface(obj, loc)-dt$avgdma8epax
mean(mm1,na.rm=T)
sd(mm1,na.rm=T)
sqrt(mean(mm1^2, na.rm=T))

obj=list(x=cx, y=cy, z=matrix((mdt$chaser+mdt$geosccm+mdt$gfdlam3+mdt$gmaonr)/4, nrow=length(cx)))
mm2=interp.surface(obj, loc)-dt$avgdma8epax
mean(mm2,na.rm=T)
sd(mm2,na.rm=T)
sqrt(mean(mm2^2, na.rm=T))


################################################################
####the following script can be ran separately for saving memory
################################################################

mdt = read.table("dma8_combined.txt",header=F,sep=" ")
names(mdt)=c('lon','lat','chaser','geosccm','gfdlam3','gmaonr','mocage','mriesm','spde', 'res', 'composite', 'fbias','fcorr')
df1=data.frame(X=mdt$lon, Y=mdt$lat, res=mdt$res, bias=mdt$fbias, composite=mdt$composite, fcorr=mdt$fcorr)


dt1 = read.table("toar-6mo8hr-dma8-2008.txt",header=F,sep=" ")
dt2 = read.table("toar-6mo8hr-dma8-2009.txt",header=F,sep=" ")
dt3 = read.table("toar-6mo8hr-dma8-2010.txt",header=F,sep=" ")
dt4 = read.table("toar-6mo8hr-dma8-2011.txt",header=F,sep=" ")
dt5 = read.table("toar-6mo8hr-dma8-2012.txt",header=F,sep=" ")
dt6 = read.table("toar-6mo8hr-dma8-2013.txt",header=F,sep=" ")
dt7 = read.table("toar-6mo8hr-dma8-2014.txt",header=F,sep=" ")
dt=rbind(dt1,dt2,dt3,dt4,dt5,dt6,dt7)
dt=na.omit(dt)
dt[dt[,4]>=100,]=99
colnames(dt)=c('id', 'lon', 'lat', 'avgdma8epax')
dt=dt[,-1]

df2=data.frame(X=dt$lon,Y=dt$lat,obs=dt$avgdma8epax)
mydist <- function(row){
  dists <- (row[["X"]] - df2$X)^2 + (row[["Y"]]- df2$Y)^2
  return(cbind(df2[which.min(dists),], distance = min(dists)))
}
z <- cbind(df1, do.call(rbind, lapply(1:nrow(df1), function(x) mydist(df1[x,])))) 


################################################################
##z = read.table("dma8_distance.txt",header=F,sep=" ")
##limit the influence of Mt Kenya
names(z)=c('lon','lat', 'res', 'fbias', 'composite','fcorr','X','Y','obs','distance')


sm=z[z$lon<=50 & z$lon>=20 & z$lat<=15 & z$lat>=-15,]

sm[sm$distance>=0.5,]$res=0
sm[sm$distance>=0.5,]$fbias=0

z[z$lon<=50 & z$lon>=20 & z$lat<=15 & z$lat>=-15,]$res= sm$res

z[z$lon<=50 & z$lon>=20 & z$lat<=15 & z$lat>=-15,]$fbias= sm$fbias

rad=z
names(rad)=c('lon','lat', 'res', 'fbias', 'composite','fcorr','X','Y','obs','distance')


png("multi_model_composite.png", height =10, width =14, units ="in", res =300)
par(mar=c(3.5, 3.5, 2, 1), mgp=c(2.4, 0.8, 0), las=1)
quilt.plot(rad$lon, rad$lat, rad$composite, xlim=c(-180,180),ylim=c(-75,75), nx=180*4, ny=75*4, xlab="Longitude",ylab="Latitude", cex.lab=1.3, horizontal=F, col=col, breaks=seq(0,100,length.out= 11))
plot(newworld, add=TRUE, lwd=1.5)
mtext('(ppb)', side=3, line=0, at=195, cex=2)
dev.off()

png("multi_model_residual.png", height =10, width =14, units ="in", res =300)
par(mar=c(3.5, 3.5, 2, 1), mgp=c(2.4, 0.8, 0), las=1)
quilt.plot(rad$lon, rad$lat, rad$res, xlim=c(-180,180),ylim=c(-75,75), nx=180*4, ny=75*4, xlab="Longitude",ylab="Latitude", cex.lab=1.3, horizontal=F, col=col2, breaks=seq(-50,50,length.out= 21))
plot(newworld, add=TRUE, lwd=1.5)
mtext('(ppb)', side=3, line=0, at=195, cex=2)
dev.off()

png("multi_model_bias.png", height =10, width =14, units ="in", res =300)
par(mar=c(3.5, 3.5, 2, 1), mgp=c(2.4, 0.8, 0), las=1)
quilt.plot(rad$lon, rad$lat, rad$fbias, xlim=c(-180,180),ylim=c(-75,75), nx=180*4, ny=75*4, xlab="Longitude",ylab="Latitude", cex.lab=1.3, horizontal=F, col=col2, breaks=seq(-50,50,length.out= 21))
plot(newworld, add=TRUE, lwd=1.5)
mtext('(ppb)', side=3, line=0, at=195, cex=2)
dev.off()


rad2=rad
rad2[rad2$distance>=2,]$res=0
rad2[rad2$distance>=2,]$fbias=0
rad2$composite= unlist(rad2$composite-rad2$res)
rad2[rad2$res==0,]$res=NA
rad2[rad2$fbias==0,]$fbias=NA
png("multi_model_bias_rad2.png", height =10, width =14, units ="in", res =300)
par(mar=c(3.5, 3.5, 2, 1), mgp=c(2.4, 0.8, 0), las=1)
quilt.plot(rad2$lon, rad2$lat, rad2$res, xlim=c(-180,180),ylim=c(-75,75), nx=180*4, ny=75*4, xlab="Longitude",ylab="Latitude", cex.lab=1.3, horizontal=F, col=col2, breaks=seq(-30,30,length.out= 21))
plot(newworld, add=TRUE, lwd=1.5)
mtext('(ppb)', side=3, line=0, at=195, cex=2)
dev.off()
png("multi_model_correction_rad2.png", height =10, width =14, units ="in", res =300)
par(mar=c(3.5, 3.5, 2, 1), mgp=c(2.4, 0.8, 0), las=1)
quilt.plot(rad2$lon, rad2$lat, rad2$composite, xlim=c(-180,180),ylim=c(-75,75), nx=180*4, ny=75*4, xlab="Longitude",ylab="Latitude", cex.lab=1.3, horizontal=F, col=col, breaks=seq(0,100,length.out= 11))
plot(newworld, add=TRUE, lwd=1.5)
rad2p=na.omit(rad2)
#points(cbind(rad2p$lon, rad2p$lat), pch='.', col='gray')
mtext('(ppb)', side=3, line=0, at=195, cex=2)
dev.off()


rad3=rad
rad3[rad3$distance>=5,]$res=0
rad3[rad3$distance>=5,]$fbias=0
rad3$composite= unlist(rad3$composite-rad3$res)
rad3[rad3$res==0,]$res=NA
rad3[rad3$fbias==0,]$fbias=NA
png("multi_model_bias_rad5.png", height =10, width =14, units ="in", res =300)
par(mar=c(3.5, 3.5, 2, 1), mgp=c(2.4, 0.8, 0), las=1)
quilt.plot(rad3$lon, rad3$lat, rad3$res, xlim=c(-180,180),ylim=c(-75,75), nx=180*4, ny=75*4, xlab="Longitude",ylab="Latitude", cex.lab=1.3, horizontal=F, col=col2, breaks=seq(-30,30,length.out= 21))
plot(newworld, add=TRUE, lwd=1.5)
mtext('(ppb)', side=3, line=0, at=195, cex=2)
dev.off()
png("multi_model_correction_rad5.png", height =10, width =14, units ="in", res =300)
par(mar=c(3.5, 3.5, 2, 1), mgp=c(2.4, 0.8, 0), las=1)
quilt.plot(rad3$lon, rad3$lat, rad3$composite, xlim=c(-180,180),ylim=c(-75,75), nx=180*4, ny=75*4, xlab="Longitude",ylab="Latitude", cex.lab=1.3, horizontal=F, col=col, breaks=seq(0,100,length.out= 11))
plot(newworld, add=TRUE, lwd=1.5)
mtext('(ppb)', side=3, line=0, at=195, cex=2)
dev.off()

rad4=rad
rad4[rad4$distance>=10,]$res=0
rad4[rad4$distance>=10,]$fbias=0
rad4$composite= unlist(rad4$composite-rad4$res)
rad4[rad4$res==0,]$res=NA
rad4[rad4$fbias==0,]$fbias=NA
png("multi_model_bias_rad10.png", height =10, width =14, units ="in", res =300)
par(mar=c(3.5, 3.5, 2, 1), mgp=c(2.4, 0.8, 0), las=1)
quilt.plot(rad4$lon, rad4$lat, rad4$res, xlim=c(-180,180),ylim=c(-75,75), nx=180*4, ny=75*4, xlab="Longitude",ylab="Latitude", cex.lab=1.3, horizontal=F, col=col2, breaks=seq(-30,30,length.out= 21))
plot(newworld, add=TRUE, lwd=1.5)
mtext('(ppb)', side=3, line=0, at=195, cex=2)
dev.off()
png("multi_model_correction_rad10.png", height =10, width =14, units ="in", res =300)
par(mar=c(3.5, 3.5, 2, 1), mgp=c(2.4, 0.8, 0), las=1)
quilt.plot(rad4$lon, rad4$lat, rad4$composite, xlim=c(-180,180),ylim=c(-75,75), nx=180*4, ny=75*4, xlab="Longitude",ylab="Latitude", cex.lab=1.3, horizontal=F, col=col, breaks=seq(0,100,length.out= 11))
plot(newworld, add=TRUE, lwd=1.5)
mtext('(ppb)', side=3, line=0, at=195, cex=2)
dev.off()

rad5=rad
rad5[rad5$distance>=15,]$res=0
rad5[rad5$distance>=15,]$fbias=0
rad5$composite= unlist(rad5$composite-rad5$res)
rad5[rad5$res==0,]$res=NA
rad5[rad5$fbias==0,]$fbias=NA
png("multi_model_bias_rad15.png", height =10, width =14, units ="in", res =300)
par(mar=c(3.5, 3.5, 2, 1), mgp=c(2.4, 0.8, 0), las=1)
quilt.plot(rad5$lon, rad5$lat, rad5$res, xlim=c(-180,180),ylim=c(-75,75), nx=180*4, ny=75*4, xlab="Longitude",ylab="Latitude", cex.lab=1.3, horizontal=F, col=col2, breaks=seq(-30,30,length.out= 21))
plot(newworld, add=TRUE, lwd=1.5)
mtext('(ppb)', side=3, line=0, at=195, cex=2)
dev.off()
png("multi_model_correction_rad15.png", height =10, width =14, units ="in", res =300)
par(mar=c(3.5, 3.5, 2, 1), mgp=c(2.4, 0.8, 0), las=1)
quilt.plot(rad5$lon, rad5$lat, rad5$composite, xlim=c(-180,180),ylim=c(-75,75), nx=180*4, ny=75*4, xlab="Longitude",ylab="Latitude", cex.lab=1.3, horizontal=F, col=col, breaks=seq(0,100,length.out= 11))
plot(newworld, add=TRUE, lwd=1.5)
mtext('(ppb)', side=3, line=0, at=195, cex=2)
dev.off()




################################################################
dt1 = read.table("toar-6mo8hr-dma8-2008.txt",header=F,sep=" ")
dt2 = read.table("toar-6mo8hr-dma8-2009.txt",header=F,sep=" ")
dt3 = read.table("toar-6mo8hr-dma8-2010.txt",header=F,sep=" ")
dt4 = read.table("toar-6mo8hr-dma8-2011.txt",header=F,sep=" ")
dt5 = read.table("toar-6mo8hr-dma8-2012.txt",header=F,sep=" ")
dt6 = read.table("toar-6mo8hr-dma8-2013.txt",header=F,sep=" ")
dt7 = read.table("toar-6mo8hr-dma8-2014.txt",header=F,sep=" ")
dt=rbind(dt1,dt2,dt3,dt4,dt5,dt6,dt7)
dt=na.omit(dt)
#dt[dt[,4]>=100,]=99
colnames(dt)=c('id', 'lon', 'lat', 'avgdma8epax')
dt=aggregate(avgdma8epax~lon+lat,data=dt,FUN=mean, na.rm=F)

mmdt=data.frame(cbind(rad$lon,rad$lat,rad$composite,rad2$composite,rad3$composite,rad4$composite,rad5$composite))
names(mmdt)=c('lon','lat','rad0', 'rad2', 'rad5', 'rad10','rad15')
cx=seq(-180,179.875,by=0.125)
cy=seq(-60,75,by=0.125)
sg=expand.grid(lon=cx, lat=cy)
mmdt = merge(mmdt, sg,by=c("lat","lon"), all=T, sort=T)

obj=list(x=cx, y=cy, z=matrix(mmdt$rad0, nrow=length(cx)))
loc=cbind(x=dt$lon, y=dt$lat)
interp.surface(obj, loc)-> back
sqrt(mean((back-dt$avgdma8epax)^2, na.rm=T))

obj=list(x=cx, y=cy, z=matrix(mmdt$rad2, nrow=length(cx)))
interp.surface(obj, loc)-> back
sqrt(mean((back-dt$avgdma8epax)^2, na.rm=T))

obj=list(x=cx, y=cy, z=matrix(mmdt$rad5, nrow=length(cx)))
interp.surface(obj, loc)-> back
sqrt(mean((back-dt$avgdma8epax)^2, na.rm=T))

obj=list(x=cx, y=cy, z=matrix(mmdt$rad10, nrow=length(cx)))
interp.surface(obj, loc)-> back
sqrt(mean((back-dt$avgdma8epax)^2, na.rm=T))

obj=list(x=cx, y=cy, z=matrix(mmdt$rad15, nrow=length(cx)))
interp.surface(obj, loc)-> back
sqrt(mean((back-dt$avgdma8epax)^2, na.rm=T))


##############################
####save as netcdf
##############################
cx=seq(-180,179.875,by=0.125)
cy=seq(-60,75,by=0.125)
obj=list(x=cx, y=cy, z=matrix(mmdt$rad2, nrow=length(cx)))
cx=seq(-179.95,179.95,by=0.1)
cy=seq(-60.05,74.95,by=0.1)
sg=expand.grid(lon=cx, lat=cy)
mmdt=interp.surface(obj, sg)

library(ncdf4)
filename="dma8_multimodel_2radius-corrected.nc"

xvals <- cx
yvals <- cy
nx <- length(xvals)
ny <- length(yvals)
lon1 <- ncdim_def("longitude", "degrees_east", xvals)
lat2 <- ncdim_def("latitude", "degrees_north", yvals)

#time <- ncdim_def("Time","months", 1:12, unlim=TRUE)
mv <- -999 #missing value to use
var_temp <- ncvar_def("DMA8", "ppb", list(lon1, lat2), longname="DMA8_2008_2014 (lon:-179.95,179.95,by=0.1; -60.05,74.95,by=0.1)", mv)
ncnew <- nc_create(filename, list(var_temp))

print(paste("The file has", ncnew$nvars,"variables"))
print(paste("The file has", ncnew$ndim,"dimensions"))

# data
data <- unlist(as.vector(mmdt))
ncvar_put(ncnew, var_temp, data, start=c(1,1), count=c(nx,ny))

# Don't forget to close the file
nc_close(ncnew)

##############################
#### check the netcdf 
##############################
library(ncdf4)
library(RColorBrewer)
library(lattice)

ncname <- "dma8_multimodel_2radius-corrected"
ncfname <- paste(ncname, ".nc", sep = "")
dname <- "DMA8"
ncin <- nc_open(ncfname)
print(ncin)

ots.array <- ncvar_get(ncin, dname)
dim(ots.array)

