##miscellaneous codes for maps
##Citation: Chang, K.-L., Cooper, O.R., West, J.J., Serre, M.L., Schultz, M.G.,
##Lin, M., Marécal, V., Josse, B., Deushi, M., Sudo, K., Liu, J., and Keller, C.A.:
##A new method (M3Fusion v1) for combining observations and multiple model output
##for an improved estimate of the global surface ozone distribution,
##Geosci. Model Dev., 2019.

col=c(rgb(0.2081, 0.1663, 0.5292),
rgb(0.3961, 0.3176, 0.8000),
rgb(0.0123, 0.4213, 0.8802),
rgb(0.4941, 0.7647, 0.8980),
rgb(0.1157, 0.7022, 0.6843),
rgb(0.5216, 0.6980, 0.1725),
rgb(0.9968, 0.7513, 0.2325),
rgb(1.0000, 0.4863, 0.0000),
rgb(0.8000, 0.3176, 0.3176),
rgb(0.6980, 0.1725, 0.1725))
N=10

library(INLA)
library(fields)
library(mgcv)
library(rworldmap)
library(rgeos)
library(maptools)
library(limSolve)
library(cleangeo)  ## For clgeo_Clean()
sPDF = getMap()
sPDF = clgeo_Clean(sPDF)  ## Needed to fix up some non-closed polygons
cont = sapply(levels(sPDF$continent),
FUN = function(i) {
    ## Merge polygons within a continent
    poly = gUnionCascaded(subset(sPDF, continent==i))
    ## Give each polygon a unique ID
    poly = spChFIDs(poly, i)
    ## Make SPDF from SpatialPolygons object
    SpatialPolygonsDataFrame(poly,
    data.frame(continent=i, row.names=i))
},
USE.NAMES=TRUE)

## Bind the 6 continent-level SPDFs into a single SPDF
cont = Reduce(spRbind, cont)
newworld =getMap(resolution = "li")


library(dichromat)
col2=colorschemes$DarkRedtoBlue.18
col2=c('blue 4',col2,'red 4')


ccmi = read.table("GMAO-NR-dma8-2014.txt",header=F,sep=" ")
ccmi=as.matrix(ccmi)
lat = seq(-90, by=0.125, length=1441)
lon = seq(-180, by=0.125, length=2880)
out.p = list(x=lon, y=lat, z=ccmi)
library(maptools)
library(raster)
library(rgeos)
data(wrld_simpl)
x = out.p
r = raster(image2Grid(x))
p = as(extent(r), "SpatialPolygons")
wmap = gIntersection(wrld_simpl, p)
oceanmap = gDifference(p, wmap)

######################################################
dt1 = read.table("toar-6mo8hr-dma8-2008.txt",header=F,sep=" ")
dt2 = read.table("toar-6mo8hr-dma8-2009.txt",header=F,sep=" ")
dt3 = read.table("toar-6mo8hr-dma8-2010.txt",header=F,sep=" ")
dt4 = read.table("toar-6mo8hr-dma8-2011.txt",header=F,sep=" ")
dt5 = read.table("toar-6mo8hr-dma8-2012.txt",header=F,sep=" ")
dt6 = read.table("toar-6mo8hr-dma8-2013.txt",header=F,sep=" ")
dt7 = read.table("toar-6mo8hr-dma8-2014.txt",header=F,sep=" ")
dt=rbind(dt1,dt2,dt3,dt4,dt5,dt6,dt7)
dt=na.omit(dt)
colnames(dt)=c('station_id', 'station_lon', 'station_lat', 'avgdma8epax')

##Fig. 2
png("SET1_warm_dma8_obs_2008-14text.png", height =10, width =14, units ="in", res =300)
par(mar=c(3.5, 3.5, 2, 1), mgp=c(2.4, 0.8, 0), las=1)
quilt.plot(dt$station_lon, dt$station_lat, dt$avgdma8epax, nx=180, ny=90, xlab="Longitude", ylab="Latitude", horizontal=F, col=col, breaks=seq(0, 100, length.out= N+1), pch=20, xlim=c(-180,180), ylim=c(-75,75))
plot(newworld, add=TRUE, lwd=1.5, col='white', border='gray')
quilt.plot(dt$station_lon, dt$station_lat, dt$avgdma8epax, nx=180, ny=90, xlab="Longitude", ylab="Latitude", horizontal=F, col=col, breaks=seq(0, 100, length.out= N+1), pch=20, add=T, na.rm=T, xlim=c(-180,180), ylim=c(-75,75))
text(195,72.5,'(ppb)', cex=2)
abline(h = 75)
abline(h = -75)

segments(-168,20,-168,75,col='black')
segments(-168,75,-55,75,col='black')
segments(-55,20,-55,75,col='black')
segments(-168,15,-168,20,col='black')
segments(-168,15,-90,15 ,col='black')
text(-140, 25, "N America", cex=2)

segments(-90,20,-90,-63,col='black')
segments(-90,20,-30,20,col='black')
segments(-90,-63,-30,-63,col='black')
segments(-30,20,-30,-63,col='black')
text(-63, -58, "S America", cex=2)

segments(90,10,90,50,col='black')
segments(90,10,165,10 ,col='black')
segments(90,50,165,50,col='black')
segments(165,10,165,50,col='black')
text(150, 15, "E Asia", cex=2)

segments(30,10,30,50,col='black')
segments(30,10,90,10,col='black')
segments(30,50,90,50,col='black')
segments(90,10,90,50,col='black')
text(55, 15, "S/C Asia", cex=2)

segments(-23,37,-23,75,col='black')
segments(-23,37,30,37 ,col='black')
segments(-23,75,30,75,col='black')
segments(30,37,30,75,col='black')
text(0, 70, "Europe", cex=2)

segments(30,50,30,75,col='black')
segments(30,50,180,50,col='black')
segments(30,75,180,75,col='black')
segments(180,50,180,75,col='black')
segments(-180,50,-180,75,col='black')
segments(-180,50,-170,50,col='black')
segments(-180,75,-170,75,col='black')
segments(-170,50,-170,75,col='black')
text(100, 70, "Russia", cex=2)

segments(-20,10,-20,37,col='black')
segments(-20,37,30,37,col='black')
segments(30,10,30,37,col='black')
segments(-20,-40,-20,10,col='black')
segments(-20,-40,60,-40,col='black')
segments(60,-40,60,10,col='black') 
text(-5, -35, "Africa", cex=2)

segments(90,-50,90,10,col='black')
segments(90,10,180,10,col='black')
segments(90,-50,180,-50,col='black')
segments(-180,-50,-170,-50,col='black')
segments(-180,10,-170,10,col='black')
segments(-170,10,-170,-50,col='black') 
text(110, -45, "Oceania", cex=2)
dev.off()



##########################################################
#### supplemental plots
##########################################################

ccmi08 = read.table("CCMI-GEOSCCM-warm-dma8-2008.txt",header=F,sep=" ")
ccmi09 = read.table("CCMI-GEOSCCM-warm-dma8-2009.txt",header=F,sep=" ")
ccmi10 = read.table("CCMI-GEOSCCM-warm-dma8-2010.txt",header=F,sep=" ")
ccmi11 = read.table("CCMI-GEOSCCM-warm-dma8-2011.txt",header=F,sep=" ")
ccmi12 = read.table("CCMI-GEOSCCM-warm-dma8-2012.txt",header=F,sep=" ")
ccmi13 = read.table("CCMI-GEOSCCM-warm-dma8-2013.txt",header=F,sep=" ")
ccmi14 = read.table("CCMI-GEOSCCM-warm-dma8-2014.txt",header=F,sep=" ")
ccmi=(ccmi08+ccmi09+ccmi10+ccmi11+ccmi12+ccmi13+ccmi14)/7
ccmi=as.matrix(ccmi)
geosccm=ccmi
out.p = list(x=lon, y=lat, z=ccmi)
png("ccmi-geosccm_warm_dma8_2008-14.png", height =10, width =14, units ="in", res =300)
par(mar=c(3.5, 3.5, 2, 1), mgp=c(2.4, 0.8, 0), las=1)
image.plot(out.p, xlab="Longitude", ylab="Latitude", horizontal=F, col=col, breaks=seq(0,100, length.out= N+1), xlim=c(-180,180),ylim=c(-75,75))
##contour(out.p, drawlabels=FALSE, add=TRUE, col=24)
plot(newworld, add=TRUE, lwd=1.5)
plot(oceanmap, add = TRUE, col="white")
mtext('(ppb)', side=3, line=0, at=195, cex=2)
dev.off()

png("ccmi-geosccm_warm_dma8_contour.png", height =10, width =14, units ="in", res =300)
par(mar=c(3.5, 3.5, 2, 1), mgp=c(2.4, 0.8, 0), las=1)
contour(x=lon, y=lat, z=ccmi, xlim=c(-165,165),ylim=c(-75,75), nlevels=10, col=2, zlim = c(0,100), drawlabels=F,lwd=2)
plot(oceanmap, add = TRUE, col="white",lwd=2)
dev.off()


spde08 = read.table("interpolation-mean-warm-dma8-2008.txt",header=F,sep=" ")
spde09 = read.table("interpolation-mean-warm-dma8-2009.txt",header=F,sep=" ")
spde10 = read.table("interpolation-mean-warm-dma8-2010.txt",header=F,sep=" ")
spde11 = read.table("interpolation-mean-warm-dma8-2011.txt",header=F,sep=" ")
spde12 = read.table("interpolation-mean-warm-dma8-2012.txt",header=F,sep=" ")
spde13 = read.table("interpolation-mean-warm-dma8-2013.txt",header=F,sep=" ")
spde14 = read.table("interpolation-mean-warm-dma8-2014.txt",header=F,sep=" ")
spde=(spde08+spde09+spde10+spde11+spde12+spde13+spde14)/7
spde=as.matrix(spde)
out.p = list(x=lon, y=lat, z=spde)
png("SET1_warm_global_dma8_2008-14.png", height =10, width =14, units ="in", res =300)
par(mar=c(3.5, 3.5, 2, 1), mgp=c(2.4, 0.8, 0), las=1)
image.plot(out.p, xlab="Longitude", ylab="Latitude", horizontal=F, col=col, breaks=seq(0, 100, length.out= N+1), xlim=c(-180,180),ylim=c(-75,75))
#contour(out.p, drawlabels=FALSE, add=TRUE, col=24)
plot(newworld, add=TRUE, lwd=1.5)
plot(oceanmap, add = TRUE, col="white")
mtext('(ppb)', side=3, line=0, at=195, cex=2)
dev.off()



out.p = list(x=lon, y=lat, z=ccmi-spde)
png("diff_geosccm_warm_dma8_2008-14.png", height =10, width =14, units ="in", res =300)
par(mar=c(3.5, 3.5, 2, 1), mgp=c(2.4, 0.8, 0), las=1)
image.plot(out.p, xlab="Longitude", ylab="Latitude", horizontal=F, col=col2, breaks=seq(-50,50, length.out= 21), xlim=c(-180,180),ylim=c(-75,75))
#contour(out.p, drawlabels=FALSE, add=TRUE, col=24)
plot(newworld, add=TRUE, lwd=1.5)
plot(oceanmap, add = TRUE, col="white")
mtext('(ppb)', side=3, line=0, at=195, cex=2)
dev.off()

##CHASER
ccmi08 = read.table("CCMI-CHASER-warm-dma8-2008.txt",header=F,sep=" ")
ccmi09 = read.table("CCMI-CHASER-warm-dma8-2009.txt",header=F,sep=" ")
ccmi10 = read.table("CCMI-CHASER-warm-dma8-2010.txt",header=F,sep=" ")
ccmi11 = read.table("CCMI-CHASER-warm-dma8-2011.txt",header=F,sep=" ")
ccmi12 = read.table("CCMI-CHASER-warm-dma8-2012.txt",header=F,sep=" ")
ccmi13 = read.table("CCMI-CHASER-warm-dma8-2013.txt",header=F,sep=" ")
ccmi14 = read.table("CCMI-CHASER-warm-dma8-2014.txt",header=F,sep=" ")
ccmi=(ccmi08+ccmi09+ccmi10+ccmi11+ccmi12+ccmi13+ccmi14)/7
ccmi=as.matrix(ccmi)
chaser=ccmi
out.p = list(x=lon, y=lat, z=ccmi)
png("ccmi-chaser_warm_dma8_2008-14.png", height =10, width =14, units ="in", res =300)
par(mar=c(3.5, 3.5, 2, 1), mgp=c(2.4, 0.8, 0), las=1)
image.plot(out.p, xlab="Longitude", ylab="Latitude", horizontal=F, col=col, breaks=seq(0,100, length.out= N+1), xlim=c(-180,180),ylim=c(-75,75))
#contour(out.p, drawlabels=FALSE, add=TRUE, col=24)
plot(newworld, add=TRUE, lwd=1.5)
plot(oceanmap, add = TRUE, col="white")
mtext('(ppb)', side=3, line=0, at=195, cex=2)
dev.off()

png("ccmi-chaser_warm_dma8_contour.png", height =10, width =14, units ="in", res =300)
par(mar=c(3.5, 3.5, 2, 1), mgp=c(2.4, 0.8, 0), las=1)
contour(x=lon, y=lat, z=ccmi, xlim=c(-165,165),ylim=c(-75,75), nlevels=10, col=2, zlim = c(0,100), drawlabels=F,lwd=2)
plot(oceanmap, add = TRUE, col="white",lwd=2)
dev.off()

out.p = list(x=lon, y=lat, z=ccmi-spde)
png("diff_chaser_warm_dma8_2008-14.png", height =10, width =14, units ="in", res =300)
par(mar=c(3.5, 3.5, 2, 1), mgp=c(2.4, 0.8, 0), las=1)
image.plot(out.p, xlab="Longitude", ylab="Latitude", horizontal=F, col=col2, breaks=seq(-50,50, length.out= 21), xlim=c(-180,180),ylim=c(-75,75))
#contour(out.p, drawlabels=FALSE, add=TRUE, col=24)
plot(newworld, add=TRUE, lwd=1.5)
plot(oceanmap, add = TRUE, col="white")
mtext('(ppb)', side=3, line=0, at=195, cex=2)
dev.off()


##MOCAGE
ccmi08 = read.table("CCMI-MOCAGE-warm-dma8-2008.txt",header=F,sep=" ")
ccmi09 = read.table("CCMI-MOCAGE-warm-dma8-2009.txt",header=F,sep=" ")
ccmi10 = read.table("CCMI-MOCAGE-warm-dma8-2010.txt",header=F,sep=" ")
ccmi11 = read.table("CCMI-MOCAGE-warm-dma8-2011.txt",header=F,sep=" ")
ccmi12 = read.table("CCMI-MOCAGE-warm-dma8-2012.txt",header=F,sep=" ")
ccmi13 = read.table("CCMI-MOCAGE-warm-dma8-2013.txt",header=F,sep=" ")
ccmi14 = read.table("CCMI-MOCAGE-warm-dma8-2014.txt",header=F,sep=" ")
ccmi=(ccmi08+ccmi09+ccmi10+ccmi11+ccmi12+ccmi13+ccmi14)/7
ccmi=as.matrix(ccmi)
mocage=ccmi
out.p = list(x=lon, y=lat, z=ccmi)
png("ccmi-mocage_warm_dma8_2008-14.png", height =10, width =14, units ="in", res =300)
par(mar=c(3.5, 3.5, 2, 1), mgp=c(2.4, 0.8, 0), las=1)
image.plot(out.p, xlab="Longitude", ylab="Latitude", horizontal=F, col=col, breaks=seq(0,100, length.out= N+1), xlim=c(-180,180),ylim=c(-75,75))
#contour(out.p, drawlabels=FALSE, add=TRUE, col=24)
plot(newworld, add=TRUE, lwd=1.5)
plot(oceanmap, add = TRUE, col="white")
mtext('(ppb)', side=3, line=0, at=195, cex=2)
dev.off()

png("ccmi-mocage_warm_dma8_contour.png", height =10, width =14, units ="in", res =300)
par(mar=c(3.5, 3.5, 2, 1), mgp=c(2.4, 0.8, 0), las=1)
contour(x=lon, y=lat, z=ccmi, xlim=c(-165,165),ylim=c(-75,75), nlevels=10, col=2, zlim = c(0,100), drawlabels=F,lwd=2)
plot(oceanmap, add = TRUE, col="white",lwd=2)
dev.off()

out.p = list(x=lon, y=lat, z=ccmi-spde)
png("diff_mocage_warm_dma8_2008-14.png", height =10, width =14, units ="in", res =300)
par(mar=c(3.5, 3.5, 2, 1), mgp=c(2.4, 0.8, 0), las=1)
image.plot(out.p, xlab="Longitude", ylab="Latitude", horizontal=F, col=col2, breaks=seq(-50,50, length.out= 21), xlim=c(-180,180),ylim=c(-75,75))
#contour(out.p, drawlabels=FALSE, add=TRUE, col=24)
plot(newworld, add=TRUE, lwd=1.5)
plot(oceanmap, add = TRUE, col="white")
mtext('(ppb)', side=3, line=0, at=195, cex=2)
dev.off()


##MRIESM
ccmi08 = read.table("CCMI-MRIESM-warm-dma8-2008.txt",header=F,sep=" ")
ccmi09 = read.table("CCMI-MRIESM-warm-dma8-2009.txt",header=F,sep=" ")
ccmi10 = read.table("CCMI-MRIESM-warm-dma8-2010.txt",header=F,sep=" ")
ccmi11 = read.table("CCMI-MRIESM-warm-dma8-2011.txt",header=F,sep=" ")
ccmi12 = read.table("CCMI-MRIESM-warm-dma8-2012.txt",header=F,sep=" ")
ccmi13 = read.table("CCMI-MRIESM-warm-dma8-2013.txt",header=F,sep=" ")
ccmi14 = read.table("CCMI-MRIESM-warm-dma8-2014.txt",header=F,sep=" ")
ccmi=(ccmi08+ccmi09+ccmi10+ccmi11+ccmi12+ccmi13+ccmi14)/7
ccmi=as.matrix(ccmi)
mriesm=ccmi
out.p = list(x=lon, y=lat, z=ccmi)
png("ccmi-mriesm_warm_dma8_2008-14.png", height =10, width =14, units ="in", res =300)
par(mar=c(3.5, 3.5, 2, 1), mgp=c(2.4, 0.8, 0), las=1)
image.plot(out.p, xlab="Longitude", ylab="Latitude", horizontal=F, col=col, breaks=seq(0,100, length.out= N+1), xlim=c(-180,180),ylim=c(-75,75))
#contour(out.p, drawlabels=FALSE, add=TRUE, col=24)
plot(newworld, add=TRUE, lwd=1.5)
plot(oceanmap, add = TRUE, col="white")
mtext('(ppb)', side=3, line=0, at=195, cex=2)
dev.off()

png("ccmi-mriesm_warm_dma8_contour.png", height =10, width =14, units ="in", res =300)
par(mar=c(3.5, 3.5, 2, 1), mgp=c(2.4, 0.8, 0), las=1)
contour(x=lon, y=lat, z=ccmi, xlim=c(-165,165),ylim=c(-75,75), nlevels=10, col=2, zlim = c(0,100), drawlabels=F,lwd=2)
plot(oceanmap, add = TRUE, col="white",lwd=2)
dev.off()

out.p = list(x=lon, y=lat, z=ccmi-spde)
png("diff_mriesm_warm_dma8_2008-14.png", height =10, width =14, units ="in", res =300)
par(mar=c(3.5, 3.5, 2, 1), mgp=c(2.4, 0.8, 0), las=1)
image.plot(out.p, xlab="Longitude", ylab="Latitude", horizontal=F, col=col2, breaks=seq(-50,50, length.out= 21), xlim=c(-180,180),ylim=c(-75,75))
#contour(out.p, drawlabels=FALSE, add=TRUE, col=24)
plot(newworld, add=TRUE, lwd=1.5)
plot(oceanmap, add = TRUE, col="white")
mtext('(ppb)', side=3, line=0, at=195, cex=2)
dev.off()



##GFDLAM3
ccmi08 = read.table("CCMI-GFDLAM3-warm-dma8-2008.txt",header=F,sep=" ")
ccmi09 = read.table("CCMI-GFDLAM3-warm-dma8-2009.txt",header=F,sep=" ")
ccmi10 = read.table("CCMI-GFDLAM3-warm-dma8-2010.txt",header=F,sep=" ")
ccmi11 = read.table("CCMI-GFDLAM3-warm-dma8-2011.txt",header=F,sep=" ")
ccmi12 = read.table("CCMI-GFDLAM3-warm-dma8-2012.txt",header=F,sep=" ")
ccmi13 = read.table("CCMI-GFDLAM3-warm-dma8-2013.txt",header=F,sep=" ")
ccmi14 = read.table("CCMI-GFDLAM3-warm-dma8-2014.txt",header=F,sep=" ")
ccmi=(ccmi08+ccmi09+ccmi10+ccmi11+ccmi12+ccmi13+ccmi14)/7
ccmi=as.matrix(ccmi)
gfdlam3=ccmi
out.p = list(x=lon, y=lat, z=ccmi)
png("ccmi-gfdlam3_warm_dma8_2008-14.png", height =10, width =14, units ="in", res =300)
par(mar=c(3.5, 3.5, 2, 1), mgp=c(2.4, 0.8, 0), las=1)
image.plot(out.p, xlab="Longitude", ylab="Latitude", horizontal=F, col=col, breaks=seq(0,100, length.out= N+1), xlim=c(-180,180),ylim=c(-75,75))
#contour(out.p, drawlabels=FALSE, add=TRUE, col=24)
plot(newworld, add=TRUE, lwd=1.5)
plot(oceanmap, add = TRUE, col="white")
mtext('(ppb)', side=3, line=0, at=195, cex=2)
dev.off()

png("ccmi-gfdlam3_warm_dma8_contour.png", height =10, width =14, units ="in", res =300)
par(mar=c(3.5, 3.5, 2, 1), mgp=c(2.4, 0.8, 0), las=1)
contour(x=lon, y=lat, z=ccmi, xlim=c(-165,165),ylim=c(-75,75), nlevels=10, col=2, zlim = c(0,100), drawlabels=F,lwd=2)
plot(oceanmap, add = TRUE, col="white",lwd=2)
dev.off()

out.p = list(x=lon, y=lat, z=ccmi-spde)
png("diff_gfdlam3_warm_dma8_2008-14.png", height =10, width =14, units ="in", res =300)
par(mar=c(3.5, 3.5, 2, 1), mgp=c(2.4, 0.8, 0), las=1)
image.plot(out.p, xlab="Longitude", ylab="Latitude", horizontal=F, col=col2, breaks=seq(-50,50, length.out= 21), xlim=c(-180,180),ylim=c(-75,75))
#contour(out.p, drawlabels=FALSE, add=TRUE, col=24)
plot(newworld, add=TRUE, lwd=1.5)
plot(oceanmap, add = TRUE, col="white")
mtext('(ppb)', side=3, line=0, at=195, cex=2)
dev.off()



##G5NR-CHEM
ccmi = read.table("GMAO-NR-dma8-2014.txt",header=F,sep=" ")
ccmi=as.matrix(ccmi)
gmaonr=ccmi
out.p = list(x=lon, y=lat, z=ccmi)
png("ccmi-gmaonr_warm_dma8_2008-14.png", height =10, width =14, units ="in", res =300)
par(mar=c(3.5, 3.5, 2, 1), mgp=c(2.4, 0.8, 0), las=1)
image.plot(out.p, xlab="Longitude", ylab="Latitude", horizontal=F, col=col, breaks=seq(0,100, length.out= N+1), xlim=c(-180,180),ylim=c(-75,75))
#contour(out.p, drawlabels=FALSE, add=TRUE, col=24)
plot(newworld, add=TRUE, lwd=1.5)
plot(oceanmap, add = TRUE, col="white")
mtext('(ppb)', side=3, line=0, at=195, cex=2)
dev.off()

png("ccmi-gmaonr_warm_dma8_contour.png", height =10, width =14, units ="in", res =300)
par(mar=c(3.5, 3.5, 2, 1), mgp=c(2.4, 0.8, 0), las=1)
contour(x=lon, y=lat, z=ccmi, xlim=c(-165,165),ylim=c(-75,75), nlevels=10, col=2, zlim = c(0,100), drawlabels=F,lwd=2)
plot(oceanmap, add = TRUE, col="white",lwd=2)
dev.off()


out.p = list(x=lon, y=lat, z=ccmi-spde)
png("diff_gmaonr_warm_dma8_2008-14.png", height =10, width =14, units ="in", res =300)
par(mar=c(3.5, 3.5, 2, 1), mgp=c(2.4, 0.8, 0), las=1)
image.plot(out.p, xlab="Longitude", ylab="Latitude", horizontal=F, col=col2, breaks=seq(-50,50, length.out= 21), xlim=c(-180,180),ylim=c(-75,75))
#contour(out.p, drawlabels=FALSE, add=TRUE, col=24)
plot(newworld, add=TRUE, lwd=1.5)
plot(oceanmap, add = TRUE, col="white")
#mtext('(ppb)', side=3, line=0, at=195, cex=2)
dev.off()


out.p = list(x=lon, y=lat, z=(gfdlam3+gmaonr+geosccm+mriesm+mocage+chaser)/6)
png("multi_model_average.png", height =10, width =14, units ="in", res =300)
par(mar=c(3.5, 3.5, 2, 1), mgp=c(2.4, 0.8, 0), las=1)
image.plot(out.p, xlab="Longitude", ylab="Latitude", horizontal=F, col=col, breaks=seq(0, 100, length.out= 11), xlim=c(-180,180),ylim=c(-75,75))
#contour(out.p, drawlabels=FALSE, add=TRUE, col=24)
plot(newworld, add=TRUE, lwd=1.5)
plot(oceanmap, add = TRUE, col="white")
mtext('(ppb)', side=3, line=0, at=195, cex=2)
dev.off()


uncertainty = apply(cbind(as.vector(gfdlam3),as.vector(gmaonr),as.vector(geosccm),as.vector(mriesm),as.vector(mocage),as.vector(chaser)), 1, sd)
out.p = list(x=lon, y=lat, z=matrix(uncertainty, nrow=2880))
png("multi_model_average_sd.png", height =10, width =14, units ="in", res =300)
par(mar=c(3.5, 3.5, 2, 1), mgp=c(2.4, 0.8, 0), las=1)
image.plot(out.p, xlab="Longitude", ylab="Latitude", horizontal=F, col=col2[11:20],breaks=seq(0,25,length.out= 11), xlim=c(-180,180),ylim=c(-75,75))
#contour(out.p, drawlabels=FALSE, add=TRUE, col=24)
plot(newworld, add=TRUE, lwd=1.5)
plot(oceanmap, add = TRUE, col="white")
mtext('(ppb)', side=3, line=0, at=195, cex=2)
dev.off()





out.p = list(x=lon, y=lat, z=(gfdlam3+gmaonr+geosccm+chaser)/4)
png("multi_model_averageMM2.png", height =10, width =14, units ="in", res =300)
par(mar=c(3.5, 3.5, 2, 1), mgp=c(2.4, 0.8, 0), las=1)
image.plot(out.p, xlab="Longitude", ylab="Latitude", horizontal=F, col=col, breaks=seq(0, 100, length.out= 11), xlim=c(-180,180),ylim=c(-75,75))
#contour(out.p, drawlabels=FALSE, add=TRUE, col=24)
plot(newworld, add=TRUE, lwd=1.5)
plot(oceanmap, add = TRUE, col="white")
mtext('(ppb)', side=3, line=0, at=195, cex=2)
dev.off()


uncertainty = apply(cbind(as.vector(gfdlam3),as.vector(gmaonr),as.vector(geosccm),as.vector(chaser)), 1, sd)
out.p = list(x=lon, y=lat, z=matrix(uncertainty, nrow=2880))
png("multi_model_average_sdMM2.png", height =10, width =14, units ="in", res =300)
par(mar=c(3.5, 3.5, 2, 1), mgp=c(2.4, 0.8, 0), las=1)
image.plot(out.p, xlab="Longitude", ylab="Latitude", horizontal=F, col=col2[11:20], breaks=seq(0, 25, length.out= 11), xlim=c(-180,180),ylim=c(-75,75))
#contour(out.p, drawlabels=FALSE, add=TRUE, col=24)
plot(newworld, add=TRUE, lwd=1.5)
plot(oceanmap, add = TRUE, col="white")
mtext('(ppb)', side=3, line=0, at=195, cex=2)
dev.off()






##############################################################################################
z = read.table("dma8_output.txt",header=F,sep=" ")
names(z)=c('lon','lat','chaser','geosccm','gfdlam3','gmaonr','mocage','mriesm','spde')
z=cbind(1, z)
#AFRICA
cy1=z[z$lon>=-20,]
cy2=cy1[cy1$lon<=30,]
cy3=cy2[cy2$lat>=10,]
cy41=cy3[cy3$lat<=37,]
cy1=z[z$lon>=-20,]
cy2=cy1[cy1$lon<=60,]
cy3=cy2[cy2$lat>=-40,]
cy42=cy3[cy3$lat<=10,]
afr=rbind(cy41, cy42)
afr=na.omit(afr)
X=as.matrix(afr[,4:9])
Y=afr[,10]
qp=lsei(A=cbind(X, 1), B=Y, E=c(rep(1,6),0), F=1, G=rbind(cbind(diag(6),0),0), H=rep(0,7))
res=cbind(X,0)%*%qp$X-Y
afr$res=res


#E ASIA
cy1=z[z$lon>=90,]
cy2=cy1[cy1$lon<=165,]
cy3=cy2[cy2$lat>=10,]
cy4=cy3[cy3$lat<=50,]

ea=na.omit(cy4)
X=as.matrix(ea[,4:9])
Y=ea[,10]
qp=lsei(A=cbind(X, 1), B=Y, E=c(rep(1,6),0), F=1, G=rbind(cbind(diag(6),0),0), H=rep(0,7))
res=cbind(X,1)%*%qp$X-Y
ea$res=res


#S C ASIA
cy1=z[z$lon>=30,]
cy2=cy1[cy1$lon<=90,]
cy3=cy2[cy2$lat>=10,]
cy4=cy3[cy3$lat<=50,]

sca=na.omit(cy4)
X=as.matrix(sca[,4:9])
Y=sca[,10]
qp=lsei(A=cbind(X, 1), B=Y, E=c(rep(1,6),0), F=1, G=rbind(cbind(diag(6),0),0), H=rep(0,7))
res=cbind(X,1)%*%qp$X-Y
sca$res=res


#RUSSIA
cy1=z[z$lon>=30,]
cy2=cy1[cy1$lon<=180,]
cy3=cy2[cy2$lat>=50,]
cy41=cy3[cy3$lat<=75,]
cy1=z[z$lon>=-180,]
cy2=cy1[cy1$lon<=-170,]
cy3=cy2[cy2$lat>=50,]
cy42=cy3[cy3$lat<=75,]
rus=rbind(cy41, cy42)
rus=na.omit(rus)

X=as.matrix(rus[,4:9])
Y=rus[,10]
qp=lsei(A=cbind(X, 1), B=Y, E=c(rep(1,6),0), F=1, G=rbind(cbind(diag(6),0),0), H=rep(0,7))
res=cbind(X,1)%*%qp$X-Y
rus$res=res


#EUROPE
cy1=z[z$lon>=-23,]
cy2=cy1[cy1$lon<=30,]
cy3=cy2[cy2$lat>=37,]
cy4=cy3[cy3$lat<=75,]
eur=na.omit(cy4)
X=as.matrix(eur[,4:9])
Y=eur[,10]
qp=lsei(A=cbind(X, 1), B=Y, E=c(rep(1,6),0), F=1, G=rbind(cbind(diag(6),0),0), H=rep(0,7))
res=cbind(X,1)%*%qp$X-Y
eur$res=res



# OCEANIA 
cy1=z[z$lon>=90,]
cy2=cy1[cy1$lon<=180,]
cy3=cy2[cy2$lat>=-50,]
cy41=cy3[cy3$lat<=10,]
cy1=z[z$lon>=-180,]
cy2=cy1[cy1$lon<=-170,]
cy3=cy2[cy2$lat>=-50,]
cy42=cy3[cy3$lat<=10,]

oce=rbind(cy41, cy42)
oce=na.omit(oce)
X=as.matrix(oce[,4:9])
Y=oce[,10] 
qp=lsei(A=cbind(X, 1), B=Y, E=c(rep(1,6),0), F=1, G=rbind(cbind(diag(6),0),0), H=rep(0,7))
res=cbind(X,1)%*%qp$X-Y
oce$res=res


# N AMERICA
cy1=z[z$lon>=-168,]
cy2=cy1[cy1$lon<=-55,]
cy3=cy2[cy2$lat>=20,]
cy41=cy3[cy3$lat<=75,]
cy1=z[z$lon>=-168,]
cy2=cy1[cy1$lon<=-90,]
cy3=cy2[cy2$lat>=15,]
cy42=cy3[cy3$lat<=20,]

nam=rbind(cy41, cy42)
nam=na.omit(nam)
X=as.matrix(nam[,4:9])
Y=nam[,10] 
qp=lsei(A=cbind(X, 1), B=Y, E=c(rep(1,6),0), F=1, G=rbind(cbind(diag(6),0),0), H=rep(0,7))
res=cbind(X,1)%*%qp$X-Y
nam$res=res


# S AMERICA
cy1=z[z$lon>=-90,]
cy2=cy1[cy1$lon<=-30,]
cy3=cy2[cy2$lat>=-60,]
cy4=cy3[cy3$lat<=20,]

sam=na.omit(cy4)
X=as.matrix(sam[,4:9])
Y=sam[,10]
qp=lsei(A=cbind(X, 1), B=Y, E=c(rep(1,6),0), F=1, G=rbind(cbind(diag(6),0),0), H=rep(0,7))
res=cbind(X,1)%*%qp$X-Y
sam$res=res

comb=rbind(afr,ea,sca,rus,eur,oce,nam,sam)
png("multi_model_composite_disjoint.png", height =10, width =14, units ="in", res =300)
par(mar=c(3.5, 3.5, 2, 1), mgp=c(2.4, 0.8, 0), las=1)
quilt.plot(comb$lon, comb$lat, comb$res + comb$spde, xlim=c(-180,180),ylim=c(-75,75), nx=180*4, ny=75*4, xlab="Longitude",ylab="Latitude", cex.lab=1.3, horizontal=F)
plot(newworld, add=TRUE, lwd=1.5)
plot(oceanmap, add = TRUE, col="white")
mtext('(ppb)', side=3, line=0, at=195, cex=2)
dev.off()

